/*
 * Copyright 2019, Cypress Semiconductor Corporation or a subsidiary of 
 * Cypress Semiconductor Corporation. All Rights Reserved.
 * 
 * This software, associated documentation and materials ("Software"),
 * is owned by Cypress Semiconductor Corporation
 * or one of its subsidiaries ("Cypress") and is protected by and subject to
 * worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions.
 * Therefore, you may use this Software only as provided in the license
 * agreement accompanying the software package from which you
 * obtained this Software ("EULA").
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
 * non-transferable license to copy, modify, and compile the Software
 * source code solely for use in connection with Cypress's
 * integrated circuit products. Any reproduction, modification, translation,
 * compilation, or representation of this Software except as specified
 * above is prohibited without the express written permission of Cypress.
 *
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
 * reserves the right to make changes to the Software without notice. Cypress
 * does not assume any liability arising out of the application or use of the
 * Software or any product or circuit described in the Software. Cypress does
 * not authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death ("High Risk Product"). By
 * including Cypress's product in a High Risk Product, the manufacturer
 * of such system or application assumes all risk of such use and in doing
 * so agrees to indemnify Cypress against all liability.
 */

/** @file
 *
 * Scan Application
 *
 * Features demonstrated
 *  - WICED scan API
 *
 * This application snippet regularly scans for nearby Wi-Fi access points
 *
 * Application Instructions
 *   Connect a PC terminal to the serial port of the WICED Eval board,
 *   then build and download the application as described in the WICED
 *   Quick Start Guide
 *
 *   Each time the application scans, a list of Wi-Fi access points in
 *   range is printed to the UART
 *
 */

#include <stdlib.h>
#include "unistd.h"
#include "wiced.h"
#include "wwd_management.h"
#include "internal/wwd_sdpcm.h"
#include "internal/wwd_internal.h"
#include "network/wwd_buffer_interface.h"
#include "wiced_management.h"
#include "wiced_crypto.h"
#include "wiced.h"
#include "wiced_security_internal.h"
#include "internal/wiced_internal_api.h"
#include "besl_host.h"
#include "besl_host_interface.h"
#include "wiced_tls.h"
#include "wiced_utilities.h"
#include "wiced_supplicant.h"
#include "wiced_tls.h"
#include "wwd_events.h"
#include "wiced_log.h"
#include "wifi_utils.h"

#define SSID "Bobe"
#define USERNAME "MY_USERNAME"
#define PASSWORD "MY_PASSWORD"

static wiced_tls_session_t tls_session =
{ 0 };
static supplicant_workspace_t supplicant_workspace;
static wiced_tls_context_t context;
static wiced_tls_identity_t identity;

static int render_log_output(WICED_LOG_LEVEL_T level, char *logmsg)
{
	UNUSED_PARAMETER(level);

	write(STDOUT_FILENO, logmsg, strlen(logmsg));

	return 0;
}

void application_start()
{
	wiced_init();
	wiced_log_init(WICED_LOG_ERR, render_log_output, NULL);
	char* ssid = SSID;

	// Try to scan for this ssid
	wiced_scan_result_t ap_info;

	if (wiced_wifi_find_ap(ssid, &ap_info, NULL) != WICED_SUCCESS)
	{
		wiced_log_printf("Error finding AP for SSID %s\r\n", ssid);
		return;
	}

	besl_result_t res;
	supplicant_connection_info_t conn_info;

	// Try to load CA cert
	uint8_t *user_cert = NULL;
	uint8_t *user_key = NULL;
	uint8_t *ca_cert = NULL;
	uint32_t ca_cert_len = 0;

	memset(&context, 0, sizeof(context));

	memset(&conn_info, 0x0, sizeof(conn_info));
	conn_info.interface = WWD_STA_INTERFACE;
	conn_info.tls_identity = &identity;
	conn_info.tls_session = &tls_session;
	conn_info.context = &context;
	conn_info.trusted_ca_certificates = NULL;
	conn_info.root_ca_cert_length = 0;
	conn_info.eap_type = EAP_TYPE_PEAP;
	// conn_info.tunnel_auth_type = TUNNEL_TYPE_MSCHAPV2;
	conn_info.inner_eap_type = EAP_TYPE_MSCHAPV2;
	conn_info.auth_type = ap_info.security;
	conn_info.eap_identity = NULL;
	conn_info.user_name = USERNAME;
	conn_info.password = PASSWORD;

	wiced_log_printf("# Root CA Certs: %d\r\n", conn_info.root_ca_cert_length);
	wiced_log_printf("EAP Type: %d\r\n", conn_info.eap_type);
	wiced_log_printf("Inner EAP Type: %d\r\n", conn_info.inner_eap_type);
	wiced_log_printf("Auth Type: %d\r\n", conn_info.auth_type);
	wiced_log_printf("Username: %s\r\n", conn_info.user_name);
	wiced_log_printf("Password: %s\r\n", conn_info.password);

	wiced_log_printf("Initializing BESL supplicant\r\n");
	res = besl_supplicant_init(&supplicant_workspace, &conn_info);
	if (res == BESL_SUCCESS)
	{
		wiced_log_printf("Starting BESL supplicant\r\n");
		if (besl_supplicant_start(&supplicant_workspace) == BESL_SUCCESS)
		{
			wiced_log_printf("Initiating WiFi Join\r\n");
			wiced_result_t wifi_join_res = wwd_wifi_join_specific(&ap_info,
			NULL, 0, NULL, WWD_STA_INTERFACE);
			wiced_log_printf("WiFi Join Result: %d\r\n", wifi_join_res);
			if (wifi_join_res == WICED_SUCCESS)
			{
				wiced_log_printf("Retrieving TLS session information\r\n");
				int ret;
				if ((ret = mbedtls_ssl_get_session(
						&supplicant_workspace.tls_context->context,
						&tls_session)) != 0)
				{
					wiced_log_printf(
							" Failed to retrieve the session information %d \r\n",
							ret);
				}
				else
				{
					wiced_log_printf(
							" Successfully retrieved the session information %d \r\n",
							ret);
				}
			}
			else
			{
				res = besl_supplicant_stop(&supplicant_workspace);
				if (res != BESL_SUCCESS)
				{
					wiced_log_printf(
							"supplicant Stop failed with error = [%d]\r\n",
							res);
				}

				wiced_log_printf("De-init supplicant\r\n");
				besl_supplicant_deinit(&supplicant_workspace);
				res = wifi_join_res;

			}
		}
	}
	else
	{
		wiced_log_printf("Unable to initialize supplicant. Error = [%d]\r\n",
				res);
	}

	wiced_log_printf("After join_ent\r\n");

}
